"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _core = require("@babel/core");

var _find = _interopRequireDefault(require("@dword-design/functions/dist/find"));

var _findKey = _interopRequireDefault(require("@dword-design/functions/dist/find-key"));

var _keys = _interopRequireDefault(require("@dword-design/functions/dist/keys"));

var _replace = _interopRequireDefault(require("@dword-design/functions/dist/replace"));

var _some = _interopRequireDefault(require("@dword-design/functions/dist/some"));

var _startsWith = _interopRequireDefault(require("@dword-design/functions/dist/starts-with"));

var _babelPluginModuleResolver = require("babel-plugin-module-resolver");

var _path = _interopRequireDefault(require("path"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const isParentImport = path => /^(\.\/)?\.\.\//.test(path);

var _default = {
  meta: {
    type: 'suggestion',
    fixable: true
  },
  create: context => {
    var _babelConfig$plugins;

    const path = context.getFilename();

    const folder = _path.default.dirname(path); // can't check a non-file


    if (path === '<text>') return {};
    const manager = new _core.OptionManager();
    const babelConfig = manager.init({
      babelrc: true,
      root: folder,
      filename: path
    });
    const plugin = (_babelConfig$plugins = babelConfig.plugins, (0, _find.default)({
      key: 'module-resolver'
    })(_babelConfig$plugins));
    const options = plugin.options;
    const resolvePath = options.resolvePath || _babelPluginModuleResolver.resolvePath;
    return {
      ImportDeclaration: node => {
        var _ref, _options$alias, _importPath2, _importWithoutAlias;

        const importPath = node.source.value;
        const hasAlias = (_ref = (_options$alias = options.alias, (0, _keys.default)(_options$alias)), (0, _some.default)(alias => {
          var _importPath;

          return _importPath = importPath, (0, _startsWith.default)(`${alias}/`)(_importPath);
        })(_ref));
        const importWithoutAlias = resolvePath(importPath, path, options); // relative parent

        if (_importPath2 = importPath, isParentImport(_importPath2)) {
          var _options$alias2, _P$relative;

          const absoluteImportPath = _path.default.resolve(folder, importPath);

          const matchingAlias = (_options$alias2 = options.alias, (0, _findKey.default)((aliasPath, alias) => {
            var _absoluteImportPath;

            return _absoluteImportPath = absoluteImportPath, (0, _startsWith.default)(_path.default.resolve(folder, resolvePath(`${alias}/`, path, options)))(_absoluteImportPath);
          })(_options$alias2));

          if (!matchingAlias) {
            return context.report({
              node,
              message: `Unexpected parent import '${importPath}'. No matching alias found to fix the issue`
            });
          }

          const rewrittenImport = `${matchingAlias}/${(_P$relative = _path.default.relative(_path.default.resolve(folder, resolvePath(`${matchingAlias}/`, path, options)), absoluteImportPath), (0, _replace.default)(/\\/g, '/')(_P$relative))}`;
          return context.report({
            node,
            message: `Unexpected parent import '${importPath}'. Use '${rewrittenImport}' instead`,
            fix: fixer => fixer.replaceTextRange([node.source.range[0] + 1, node.source.range[1] - 1], rewrittenImport)
          });
        }

        if (!(_importWithoutAlias = importWithoutAlias, isParentImport(_importWithoutAlias)) && hasAlias) {
          return context.report({
            node,
            message: `Unexpected subpath import via alias '${importPath}'. Use '${importWithoutAlias}' instead`,
            fix: fixer => fixer.replaceTextRange([node.source.range[0] + 1, node.source.range[1] - 1], importWithoutAlias)
          });
        }

        return undefined;
      }
    };
  }
};
exports.default = _default;
module.exports = exports.default;